package at.ac.uibk.gitsearch.web.rest;

import java.io.IOException;
import java.io.InputStream;

import org.gitlab4j.api.GitLabApiException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.core.io.InputStreamResource;
import org.springframework.core.io.Resource;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import at.ac.uibk.gitsearch.service.GitlabService;
import at.ac.uibk.gitsearch.web.util.HeaderUtil;

/**
 * this resource should give access to various aspects of an exercise or course
 * @author Michael Breu
 *
 */
@RestController
@RequestMapping("/api")
public class ExerciseResource {
	
    private final Logger log = LoggerFactory.getLogger(ExerciseResource.class);
    
    @Autowired
    private GitlabService gitLabService;
    
    @Value("${jhipster.clientApp.name}")
    private String applicationName;


    /**
     * GET readMe : get problem statement.
     *
     * @param basketToken the basket
     * @return the ResponseEntity with status 200 (OK) and with body the exercise, or with status 404 (Not Found)
     */
    @GetMapping("/exercise/{exerciseId}")
    public ResponseEntity<Resource> getRepositoryFile(@PathVariable("exerciseId") String  exerciseId, @RequestParam("filePath") String filePath) throws IOException {
        InputStream repositoryStream = null;
        try {
            repositoryStream = gitLabService.getRepositoryFile(exerciseId, filePath);
        } catch (IOException | GitLabApiException e) {
            log.error("Cannot read file for exercise {} and path {}", exerciseId, filePath, e);
            return ResponseEntity.notFound().headers(HeaderUtil.createFailureAlert(applicationName, true, "RepositoryFile", "not found",
                    "There was an error on the server and the resource file could not be created.")).build();        }

        InputStreamResource resource = new InputStreamResource(repositoryStream);

        return ResponseEntity.ok().contentType(MediaType.APPLICATION_OCTET_STREAM).header("filename", filePath).body(resource);
    }


}
